@@ -18,6 +18,7 @@ import unittest
 
 # 3p
 from mock import (
+    call,
     mock_open,
     patch,
 )
@@ -25,10 +26,10 @@ import pytest
 
 # datadog
 from datadog import initialize, statsd
-from datadog.dogstatsd.base import DogStatsd
+from datadog import __version__ as version
+from datadog.dogstatsd.base import DogStatsd, UDP_OPTIMAL_PAYLOAD_LENGTH
 from datadog.dogstatsd.context import TimedContextManagerDecorator
 from datadog.util.compat import is_higher_py35, is_p3k
-from datadog.util.config import get_version
 from tests.util.contextmanagers import preserve_environment_variable, EnvVars
 from tests.unit.dogstatsd.fixtures import load_fixtures
 
@@ -50,9 +51,12 @@ class FakeSocket(object):
             assert type(payload) == str
         self.payloads.append(payload)
 
-    def recv(self):
+    def recv(self, n = 1):
         try:
-            return self.payloads.popleft().decode('utf-8')
+            out = []
+            for i in range(n):
+                out.append(self.payloads.popleft().decode('utf-8'))
+            return '\n'.join(out)
         except IndexError:
             return None
 
@@ -77,11 +81,10 @@ class OverflownSocket(FakeSocket):
         raise error
 
 
-def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes_dropped=0, packets_sent=0, packets_dropped=0, transport="udp", tags=""):
-    version = get_version()
+def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes_dropped=0, packets_sent=1, packets_dropped=0, transport="udp", tags=""):
     tags = "," + tags if tags else ""
 
-    return "\ndatadog.dogstatsd.client.metrics:{}|c|#client:py,client_version:{},client_transport:{}{}\n".format(metrics, version, transport, tags) \
+    return "datadog.dogstatsd.client.metrics:{}|c|#client:py,client_version:{},client_transport:{}{}\n".format(metrics, version, transport, tags) \
         + "datadog.dogstatsd.client.events:{}|c|#client:py,client_version:{},client_transport:{}{}\n".format(events, version, transport, tags) \
         + "datadog.dogstatsd.client.service_checks:{}|c|#client:py,client_version:{},client_transport:{}{}\n".format(service_checks, version, transport, tags) \
         + "datadog.dogstatsd.client.bytes_sent:{}|c|#client:py,client_version:{},client_transport:{}{}\n".format(bytes_sent, version, transport, tags) \
@@ -91,7 +94,8 @@ def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes
 
 def assert_equal_telemetry(expected_payload, actual_payload, telemetry=None):
     if telemetry is None:
-        telemetry = telemetry_metrics()
+        telemetry = telemetry_metrics(bytes_sent=len(expected_payload))
+    expected_payload += "\n"
     expected_payload += telemetry
     return assert_equal(expected_payload, actual_payload)
 
@@ -111,7 +115,7 @@ class TestDogStatsd(unittest.TestCase):
         self._procfs_mock.__enter__().return_value.readlines.return_value = route_data.split("\n")
 
     #def setup_method(self, method):
-    #    self.statsd._reset_telementry()
+    #    self.statsd._reset_telemetry()
 
     def tearDown(self):
         """
@@ -119,8 +123,11 @@ class TestDogStatsd(unittest.TestCase):
         """
         self._procfs_mock.__exit__()
 
-    def recv(self):
-        return self.statsd.socket.recv()
+    def recv(self, n=1):
+        packets = []
+        for _ in range(n):
+            packets.append(self.statsd.socket.recv())
+        return "\n".join(packets)
 
     def test_initialization(self):
         """
@@ -187,52 +194,52 @@ class TestDogStatsd(unittest.TestCase):
 
     def test_set(self):
         self.statsd.set('set', 123)
-        assert_equal_telemetry('set:123|s', self.recv())
+        assert_equal_telemetry('set:123|s', self.recv(2))
 
     def test_gauge(self):
         self.statsd.gauge('gauge', 123.4)
-        assert_equal_telemetry('gauge:123.4|g', self.recv())
+        assert_equal_telemetry('gauge:123.4|g', self.recv(2))
 
     def test_counter(self):
         self.statsd.increment('page.views')
-        assert_equal_telemetry('page.views:1|c', self.recv())
+        assert_equal_telemetry('page.views:1|c', self.recv(2))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
         self.statsd.increment('page.views', 11)
-        assert_equal_telemetry('page.views:11|c', self.recv())
+        assert_equal_telemetry('page.views:11|c', self.recv(2))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
         self.statsd.decrement('page.views')
-        assert_equal_telemetry('page.views:-1|c', self.recv())
+        assert_equal_telemetry('page.views:-1|c', self.recv(2))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
         self.statsd.decrement('page.views', 12)
-        assert_equal_telemetry('page.views:-12|c', self.recv())
+        assert_equal_telemetry('page.views:-12|c', self.recv(2))
 
     def test_histogram(self):
         self.statsd.histogram('histo', 123.4)
-        assert_equal_telemetry('histo:123.4|h', self.recv())
+        assert_equal_telemetry('histo:123.4|h', self.recv(2))
 
     def test_pipe_in_tags(self):
         self.statsd.gauge('gt', 123.4, tags=['pipe|in:tag', 'red'])
-        assert_equal_telemetry('gt:123.4|g|#pipe_in:tag,red', self.recv())
+        assert_equal_telemetry('gt:123.4|g|#pipe_in:tag,red', self.recv(2))
 
     def test_tagged_gauge(self):
         self.statsd.gauge('gt', 123.4, tags=['country:china', 'age:45', 'blue'])
-        assert_equal_telemetry('gt:123.4|g|#country:china,age:45,blue', self.recv())
+        assert_equal_telemetry('gt:123.4|g|#country:china,age:45,blue', self.recv(2))
 
     def test_tagged_counter(self):
         self.statsd.increment('ct', tags=[u'country:españa', 'red'])
-        assert_equal_telemetry(u'ct:1|c|#country:españa,red', self.recv())
+        assert_equal_telemetry(u'ct:1|c|#country:españa,red', self.recv(2))
 
     def test_tagged_histogram(self):
         self.statsd.histogram('h', 1, tags=['red'])
-        assert_equal_telemetry('h:1|h|#red', self.recv())
+        assert_equal_telemetry('h:1|h|#red', self.recv(2))
 
     def test_sample_rate(self):
         self.statsd._telemetry = False # disabling telemetry since sample_rate imply randomness
         self.statsd.increment('c', sample_rate=0)
-        assert not self.recv()
+        assert not self.statsd.socket.recv()
         for i in range(10000):
             self.statsd.increment('sampled_counter', sample_rate=0.3)
         self.assert_almost_equal(3000, len(self.statsd.socket.payloads), 150)
@@ -258,28 +265,32 @@ class TestDogStatsd(unittest.TestCase):
 
     def test_timing(self):
         self.statsd.timing('t', 123)
-        assert_equal_telemetry('t:123|ms', self.recv())
+        assert_equal_telemetry('t:123|ms', self.recv(2))
 
     def test_event(self):
         self.statsd.event('Title', u'L1\nL2', priority='low', date_happened=1375296969)
-        assert_equal_telemetry(u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low', self.recv(), telemetry=telemetry_metrics(metrics=0, events=1))
+        event = u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low'
+        assert_equal_telemetry(event, self.recv(2), telemetry=telemetry_metrics(metrics=0, events=1, bytes_sent=len(event)))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
 
         self.statsd.event('Title', u'♬ †øU †øU ¥ºu T0µ ♪',
                           aggregation_key='key', tags=['t1', 't2:v2'])
-        assert_equal_telemetry(u'_e{5,19}:Title|♬ †øU †øU ¥ºu T0µ ♪|k:key|#t1,t2:v2', self.recv(), telemetry=telemetry_metrics(metrics=0, events=1))
+        event = u'_e{5,19}:Title|♬ †øU †øU ¥ºu T0µ ♪|k:key|#t1,t2:v2'
+        assert_equal_telemetry(event, self.recv(2), telemetry=telemetry_metrics(metrics=0, events=1, bytes_sent=len(event)))
 
     def test_event_constant_tags(self):
         self.statsd.constant_tags = ['bar:baz', 'foo']
         self.statsd.event('Title', u'L1\nL2', priority='low', date_happened=1375296969)
-        assert_equal_telemetry(u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low|#bar:baz,foo', self.recv(), telemetry=telemetry_metrics(metrics=0, events=1, tags="bar:baz,foo"))
+        event = u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low|#bar:baz,foo'
+        assert_equal_telemetry(event, self.recv(2), telemetry=telemetry_metrics(metrics=0, events=1, tags="bar:baz,foo", bytes_sent=len(event)))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
 
         self.statsd.event('Title', u'♬ †øU †øU ¥ºu T0µ ♪',
                           aggregation_key='key', tags=['t1', 't2:v2'])
-        assert_equal_telemetry(u'_e{5,19}:Title|♬ †øU †øU ¥ºu T0µ ♪|k:key|#t1,t2:v2,bar:baz,foo', self.recv(), telemetry=telemetry_metrics(metrics=0, events=1, tags="bar:baz,foo"))
+        event = u'_e{5,19}:Title|♬ †øU †øU ¥ºu T0µ ♪|k:key|#t1,t2:v2,bar:baz,foo'
+        assert_equal_telemetry(event, self.recv(2), telemetry=telemetry_metrics(metrics=0, events=1, tags="bar:baz,foo", bytes_sent=len(event)))
 
     def test_service_check(self):
         now = int(time.time())
@@ -287,9 +298,8 @@ class TestDogStatsd(unittest.TestCase):
             'my_check.name', self.statsd.WARNING,
             tags=['key1:val1', 'key2:val2'], timestamp=now,
             hostname='i-abcd1234', message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪")
-        assert_equal_telemetry(
-            u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2|m:{2}'
-            .format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪"), self.recv(), telemetry=telemetry_metrics(metrics=0, service_checks=1))
+        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2|m:{2}'.format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪")
+        assert_equal_telemetry(check, self.recv(2), telemetry=telemetry_metrics(metrics=0, service_checks=1, bytes_sent=len(check)))
 
     def test_service_check_constant_tags(self):
         self.statsd.constant_tags = ['bar:baz', 'foo']
@@ -298,19 +308,19 @@ class TestDogStatsd(unittest.TestCase):
             'my_check.name', self.statsd.WARNING,
             timestamp=now,
             hostname='i-abcd1234', message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪")
-        assert_equal_telemetry(
-            u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#bar:baz,foo|m:{2}'
-            .format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪"), self.recv(), telemetry=telemetry_metrics(metrics=0, service_checks=1, tags="bar:baz,foo"))
+        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#bar:baz,foo|m:{2}'.format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪")
+        assert_equal_telemetry(check, self.recv(2),
+            telemetry=telemetry_metrics(metrics=0, service_checks=1, tags="bar:baz,foo", bytes_sent=len(check)))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
 
         self.statsd.service_check(
             'my_check.name', self.statsd.WARNING,
             tags=['key1:val1', 'key2:val2'], timestamp=now,
             hostname='i-abcd1234', message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪")
-        assert_equal_telemetry(
-            u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2,bar:baz,foo|m:{2}'
-            .format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪"), self.recv(), telemetry=telemetry_metrics(metrics=0, service_checks=1, tags="bar:baz,foo"))
+        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2,bar:baz,foo|m:{2}'.format(self.statsd.WARNING, now, u"♬ †øU \\n†øU ¥ºu|m\: T0µ ♪")
+        assert_equal_telemetry(check, self.recv(2),
+            telemetry=telemetry_metrics(metrics=0, service_checks=1, tags="bar:baz,foo", bytes_sent=len(check)))
 
     def test_metric_namespace(self):
         """
@@ -318,31 +328,35 @@ class TestDogStatsd(unittest.TestCase):
         """
         self.statsd.namespace = "foo"
         self.statsd.gauge('gauge', 123.4)
-        assert_equal_telemetry('foo.gauge:123.4|g', self.recv())
+        assert_equal_telemetry('foo.gauge:123.4|g', self.recv(2))
 
     # Test Client level contant tags
     def test_gauge_constant_tags(self):
         self.statsd.constant_tags=['bar:baz', 'foo']
         self.statsd.gauge('gauge', 123.4)
-        assert_equal_telemetry('gauge:123.4|g|#bar:baz,foo', self.recv(), telemetry=telemetry_metrics(tags="bar:baz,foo"))
+        metric = 'gauge:123.4|g|#bar:baz,foo'
+        assert_equal_telemetry(metric, self.recv(2), telemetry=telemetry_metrics(tags="bar:baz,foo", bytes_sent=len(metric)))
 
     def test_counter_constant_tag_with_metric_level_tags(self):
         self.statsd.constant_tags=['bar:baz', 'foo']
         self.statsd.increment('page.views', tags=['extra'])
-        assert_equal_telemetry('page.views:1|c|#extra,bar:baz,foo', self.recv(), telemetry=telemetry_metrics(tags="bar:baz,foo"))
+        metric = 'page.views:1|c|#extra,bar:baz,foo'
+        assert_equal_telemetry(metric, self.recv(2), telemetry=telemetry_metrics(tags="bar:baz,foo", bytes_sent=len(metric)))
 
     def test_gauge_constant_tags_with_metric_level_tags_twice(self):
         metric_level_tag = ['foo:bar']
         self.statsd.constant_tags=['bar:baz']
         self.statsd.gauge('gauge', 123.4, tags=metric_level_tag)
-        assert_equal_telemetry('gauge:123.4|g|#foo:bar,bar:baz', self.recv(), telemetry=telemetry_metrics(tags="bar:baz"))
+        metric = 'gauge:123.4|g|#foo:bar,bar:baz'
+        assert_equal_telemetry(metric, self.recv(2), telemetry=telemetry_metrics(tags="bar:baz", bytes_sent=len(metric)))
 
-        self.statsd._reset_telementry()
+        self.statsd._reset_telemetry()
 
         # sending metrics multiple times with same metric-level tags
         # should not duplicate the tags being sent
         self.statsd.gauge('gauge', 123.4, tags=metric_level_tag)
-        assert_equal_telemetry('gauge:123.4|g|#foo:bar,bar:baz', self.recv(), telemetry=telemetry_metrics(tags="bar:baz"))
+        metric = "gauge:123.4|g|#foo:bar,bar:baz"
+        assert_equal_telemetry(metric, self.recv(2), telemetry=telemetry_metrics(tags="bar:baz", bytes_sent=len(metric)))
 
     @staticmethod
     def assert_almost_equal(a, b, delta):
@@ -362,7 +376,8 @@ class TestDogStatsd(unittest.TestCase):
         with mock.patch("datadog.dogstatsd.base.log") as mock_log:
             self.statsd.gauge('no error', 1)
             mock_log.error.assert_not_called()
-            mock_log.warning.assert_called_once_with("Socket send would block: Socket error, dropping the packet")
+            c = [call("Socket send would block: Socket error, dropping the packet")]
+            mock_log.warning.assert_has_calls(c * 2)
 
     def test_timed(self):
         """
@@ -382,7 +397,7 @@ class TestDogStatsd(unittest.TestCase):
         # Assert it handles args and kwargs correctly.
         assert_equal(result, (1, 2, 1, 3))
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -399,7 +414,7 @@ class TestDogStatsd(unittest.TestCase):
 
         func(1, 2, d=3)
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -424,7 +439,7 @@ class TestDogStatsd(unittest.TestCase):
         func(1, 2, d=3)
 
         # Assess the packet
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -441,7 +456,7 @@ class TestDogStatsd(unittest.TestCase):
 
         func(1, 2, d=3)
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv()
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -467,7 +482,7 @@ class TestDogStatsd(unittest.TestCase):
         # Assert it handles args and kwargs correctly.
         assert_equal(result, (1, 2, 1, 3))
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -496,7 +511,7 @@ class TestDogStatsd(unittest.TestCase):
         loop.close()
 
         # Assert
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -513,7 +528,7 @@ class TestDogStatsd(unittest.TestCase):
             assert isinstance(timer, TimedContextManagerDecorator)
             time.sleep(0.5)
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -526,7 +541,7 @@ class TestDogStatsd(unittest.TestCase):
         with self.statsd.timed('timed_context.test', use_ms=True) as timer:
             time.sleep(0.5)
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -552,7 +567,7 @@ class TestDogStatsd(unittest.TestCase):
             func(self)
 
         # Ensure the timing was recorded.
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -572,7 +587,7 @@ class TestDogStatsd(unittest.TestCase):
             func(self)
 
         # Ensure the timing was recorded.
-        packet = self.recv()
+        packet = self.statsd.socket.recv()
         assert_equal(packet, None)
 
     def test_timed_start_stop_calls(self):
@@ -582,7 +597,7 @@ class TestDogStatsd(unittest.TestCase):
         time.sleep(0.5)
         timer.stop()
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -596,7 +611,7 @@ class TestDogStatsd(unittest.TestCase):
         time.sleep(0.5)
         timer.stop()
 
-        packet = self.recv().split("\n")[0] # ignore telemetry packet
+        packet = self.recv(2).split("\n")[0] # ignore telemetry packet
         name_value, type_ = packet.split('|')
         name, value = name_value.split(':')
 
@@ -609,8 +624,8 @@ class TestDogStatsd(unittest.TestCase):
         self.statsd.gauge('page.views', 123)
         self.statsd.timing('timer', 123)
         self.statsd.close_buffer()
-
-        assert_equal_telemetry("page.views:123|g\ntimer:123|ms", self.recv(), telemetry=telemetry_metrics(metrics=2))
+        expected = "page.views:123|g\ntimer:123|ms"
+        assert_equal_telemetry(expected, self.recv(2), telemetry=telemetry_metrics(metrics=2, bytes_sent=len(expected)))
 
     def test_telemetry(self):
         self.statsd.metrics_count = 1
@@ -625,16 +640,16 @@ class TestDogStatsd(unittest.TestCase):
         self.statsd.gauge('page.views', 123)
         self.statsd.close_buffer()
 
-        telemetry = telemetry_metrics(metrics=2, events=2, service_checks=3, bytes_sent=4,
-                                          bytes_dropped=5,  packets_sent=6, packets_dropped=7)
-
         payload = "page.views:123|g"
-        assert_equal_telemetry(payload, self.recv(), telemetry=telemetry)
+        telemetry = telemetry_metrics(metrics=2, events=2, service_checks=3, bytes_sent=4 + len(payload),
+                                          bytes_dropped=5,  packets_sent=7, packets_dropped=7)
+
+        assert_equal_telemetry(payload, self.recv(2), telemetry=telemetry)
 
         assert_equal(0, self.statsd.metrics_count)
         assert_equal(0, self.statsd.events_count)
         assert_equal(0, self.statsd.service_checks_count)
-        assert_equal(len(payload) + len(telemetry), self.statsd.bytes_sent)
+        assert_equal(len(telemetry), self.statsd.bytes_sent)
         assert_equal(0, self.statsd.bytes_dropped)
         assert_equal(1, self.statsd.packets_sent)
         assert_equal(0, self.statsd.packets_dropped)
@@ -648,14 +663,15 @@ class TestDogStatsd(unittest.TestCase):
         statsd._last_flush_time = time.time() + statsd._telemetry_flush_interval
         statsd.gauge('gauge', 123.4)
 
-        assert_equal('gauge:123.4|g', fake_socket.recv())
+        metric = 'gauge:123.4|g'
+        assert_equal(metric, fake_socket.recv())
 
         t1 = time.time()
         # setting the last flush time in the past to trigger a telemetry flush
         statsd._last_flush_time = t1 - statsd._telemetry_flush_interval -1
         statsd.gauge('gauge', 123.4)
+        assert_equal_telemetry(metric, fake_socket.recv(2), telemetry=telemetry_metrics(metrics=2, bytes_sent=2*len(metric), packets_sent=2))
 
-        assert_equal_telemetry('gauge:123.4|g', fake_socket.recv(), telemetry=telemetry_metrics(metrics=2, bytes_sent=13, packets_sent=1))
         # assert that _last_flush_time has been updated
         assert t1 < statsd._last_flush_time
 
@@ -675,7 +691,8 @@ class TestDogStatsd(unittest.TestCase):
 
         statsd.close_buffer()
 
-        assert_equal_telemetry('gauge1:1|g\ngauge2:2|g', fake_socket.recv(), telemetry=telemetry_metrics(metrics=2))
+        metric = 'gauge1:1|g\ngauge2:2|g'
+        assert_equal_telemetry(metric, fake_socket.recv(2), telemetry=telemetry_metrics(metrics=2, bytes_sent=len(metric)))
         # assert that _last_flush_time has been updated
         assert t1 < statsd._last_flush_time
 
@@ -685,24 +702,30 @@ class TestDogStatsd(unittest.TestCase):
             statsd.socket = fake_socket
             statsd.gauge('page.views', 123)
             statsd.timing('timer', 123)
-
-        assert_equal_telemetry("page.views:123|g\ntimer:123|ms", fake_socket.recv(), telemetry=telemetry_metrics(metrics=2))
+        metric = "page.views:123|g\ntimer:123|ms"
+        assert_equal(metric, fake_socket.recv())
+        assert_equal(telemetry_metrics(metrics=2, bytes_sent=len(metric)), fake_socket.recv())
+        # assert_equal_telemetry("page.views:123|g\ntimer:123|ms", fake_socket.recv(2), telemetry=telemetry_metrics(metrics=2))
 
     def test_batched_buffer_autoflush(self):
         fake_socket = FakeSocket()
         bytes_sent = 0
         with DogStatsd(telemetry_min_flush_interval=0) as statsd:
+            single_metric = 'mycounter:1|c'
+            assert_equal(statsd._max_payload_size, UDP_OPTIMAL_PAYLOAD_LENGTH)
+            metrics_per_packet = statsd._max_payload_size // (len(single_metric) + 1)
             statsd.socket = fake_socket
-            for i in range(51):
+            for i in range(metrics_per_packet + 1):
                 statsd.increment('mycounter')
-            payload = '\n'.join(['mycounter:1|c' for i in range(50)])
+            payload = '\n'.join([single_metric for i in range(metrics_per_packet)])
 
-            telemetry = telemetry_metrics(metrics=50)
-            bytes_sent += len(payload)+len(telemetry)
-
-            assert_equal_telemetry(payload, fake_socket.recv(), telemetry=telemetry)
-
-        assert_equal_telemetry('mycounter:1|c', fake_socket.recv(), telemetry=telemetry_metrics(packets_sent=1, bytes_sent=bytes_sent))
+            telemetry = telemetry_metrics(metrics=metrics_per_packet+1, bytes_sent=len(payload))
+            bytes_sent += len(payload) + len(telemetry)
+            assert_equal(payload, fake_socket.recv())
+            assert_equal(telemetry, fake_socket.recv())
+        assert_equal(single_metric, fake_socket.recv())
+        telemetry = telemetry_metrics(metrics=0, packets_sent=2, bytes_sent=len(single_metric) + len(telemetry))
+        assert_equal(telemetry, fake_socket.recv())
 
     def test_module_level_instance(self):
         assert isinstance(statsd, DogStatsd)
@@ -731,18 +754,20 @@ class TestDogStatsd(unittest.TestCase):
             statsd = DogStatsd(telemetry_min_flush_interval=0)
         statsd.socket = FakeSocket()
         statsd.gauge('gt', 123.4)
-        assert_equal_telemetry('gt:123.4|g|#country:china,age:45,blue',
-                statsd.socket.recv(),
-                telemetry=telemetry_metrics(tags="country:china,age:45,blue"))
+        metric = 'gt:123.4|g|#country:china,age:45,blue'
+        assert_equal(metric, statsd.socket.recv())
+        assert_equal(telemetry_metrics(tags="country:china,age:45,blue", bytes_sent=len(metric)), statsd.socket.recv())
 
     def test_tags_from_environment_and_constant(self):
         with preserve_environment_variable('DATADOG_TAGS'):
-           os.environ['DATADOG_TAGS'] = 'country:china,age:45,blue'
-           statsd = DogStatsd(constant_tags=['country:canada', 'red'], telemetry_min_flush_interval=0)
+            os.environ['DATADOG_TAGS'] = 'country:china,age:45,blue'
+            statsd = DogStatsd(constant_tags=['country:canada', 'red'], telemetry_min_flush_interval=0)
         statsd.socket = FakeSocket()
         statsd.gauge('gt', 123.4)
         tags="country:canada,red,country:china,age:45,blue"
-        assert_equal_telemetry('gt:123.4|g|#'+tags, statsd.socket.recv(), telemetry=telemetry_metrics(tags=tags))
+        metric = 'gt:123.4|g|#'+tags
+        assert_equal(metric, statsd.socket.recv())
+        assert_equal(telemetry_metrics(tags=tags, bytes_sent=len(metric)), statsd.socket.recv())
 
     def test_entity_tag_from_environment(self):
         with preserve_environment_variable('DD_ENTITY_ID'):
@@ -750,9 +775,11 @@ class TestDogStatsd(unittest.TestCase):
             statsd = DogStatsd(telemetry_min_flush_interval=0)
         statsd.socket = FakeSocket()
         statsd.gauge('gt', 123.4)
-        assert_equal_telemetry('gt:123.4|g|#dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d',
-                statsd.socket.recv(),
-                telemetry=telemetry_metrics(tags="dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d"))
+        metric = 'gt:123.4|g|#dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d'
+        assert_equal(metric, statsd.socket.recv())
+        assert_equal(
+            telemetry_metrics(tags="dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d", bytes_sent=len(metric)),
+            statsd.socket.recv())
 
     def test_entity_tag_from_environment_and_constant(self):
         with preserve_environment_variable('DD_ENTITY_ID'):
@@ -760,9 +787,11 @@ class TestDogStatsd(unittest.TestCase):
             statsd = DogStatsd(constant_tags=['country:canada', 'red'], telemetry_min_flush_interval=0)
         statsd.socket = FakeSocket()
         statsd.gauge('gt', 123.4)
-        assert_equal_telemetry('gt:123.4|g|#country:canada,red,dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d',
-                statsd.socket.recv(),
-                telemetry=telemetry_metrics(tags="country:canada,red,dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d"))
+        metric = 'gt:123.4|g|#country:canada,red,dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d'
+        assert_equal(metric, statsd.socket.recv())
+        assert_equal(
+            telemetry_metrics(tags="country:canada,red,dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d",
+            bytes_sent=len(metric)), statsd.socket.recv())
 
     def test_entity_tag_and_tags_from_environment_and_constant(self):
         with preserve_environment_variable('DATADOG_TAGS'):
@@ -773,7 +802,9 @@ class TestDogStatsd(unittest.TestCase):
         statsd.socket = FakeSocket()
         statsd.gauge('gt', 123.4)
         tags = "country:canada,red,country:china,age:45,blue,dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d"
-        assert_equal_telemetry('gt:123.4|g|#'+tags, statsd.socket.recv(), telemetry=telemetry_metrics(tags=tags))
+        metric = 'gt:123.4|g|#'+tags
+        assert_equal(metric, statsd.socket.recv())
+        assert_equal(telemetry_metrics(tags=tags, bytes_sent=len(metric)), statsd.socket.recv())
 
     def test_dogstatsd_initialization_with_dd_env_service_version(self):
         """
@@ -813,40 +844,37 @@ class TestDogStatsd(unittest.TestCase):
             # Make call with no tags passed; only the globally configured tags will be used.
             global_tags_str = ','.join([t for t in global_tags])
             statsd.gauge('gt', 123.4)
-            assert_equal_telemetry(
-                # Protect against the no tags case.
-                'gt:123.4|g|#{}'.format(global_tags_str) if global_tags_str else 'gt:123.4|g',
-                statsd.socket.recv(),
-                telemetry=telemetry_metrics(tags=global_tags_str)
-            )
-            statsd._reset_telementry()
+            # Protect against the no tags case.
+            metric = 'gt:123.4|g|#{}'.format(global_tags_str) if global_tags_str else 'gt:123.4|g'
+            assert_equal(metric, statsd.socket.recv())
+            assert_equal(telemetry_metrics(tags=global_tags_str, bytes_sent=len(metric)), statsd.socket.recv())
+            statsd._reset_telemetry()
 
             # Make another call with local tags passed.
             passed_tags = ['env:prod', 'version:def456', 'custom_tag:toad']
             all_tags_str = ','.join([t for t in passed_tags + global_tags])
             statsd.gauge('gt', 123.4, tags=passed_tags)
-            assert_equal_telemetry(
-                'gt:123.4|g|#{}'.format(all_tags_str),
-                statsd.socket.recv(),
-                telemetry=telemetry_metrics(tags=global_tags_str)
-            )
+
+            metric = 'gt:123.4|g|#{}'.format(all_tags_str)
+            assert_equal(metric, statsd.socket.recv())
+            assert_equal(telemetry_metrics(tags=global_tags_str, bytes_sent=len(metric)), statsd.socket.recv())
 
     def test_gauge_doesnt_send_None(self):
         self.statsd.gauge('metric', None)
-        assert self.recv() is None
+        assert self.statsd.socket.recv() is None
 
     def test_increment_doesnt_send_None(self):
         self.statsd.increment('metric', None)
-        assert self.recv() is None
+        assert self.statsd.socket.recv() is None
 
     def test_decrement_doesnt_send_None(self):
         self.statsd.decrement('metric', None)
-        assert self.recv() is None
+        assert self.statsd.socket.recv() is None
 
     def test_timing_doesnt_send_None(self):
         self.statsd.timing('metric', None)
-        assert self.recv() is None
+        assert self.statsd.socket.recv() is None
 
     def test_histogram_doesnt_send_None(self):
         self.statsd.histogram('metric', None)
-        assert self.recv() is None
+        assert self.statsd.socket.recv() is None
